use maud::{Markup, Render, html};

use crate::ui::UIWidget;

pub struct ZIndex(Box<dyn UIWidget>, u8);

impl Render for ZIndex {
    fn render(&self) -> Markup {
        self.render_with_class("")
    }
}

impl ZIndex {
    pub fn auto<T: UIWidget + 'static>(inner: T) -> Self {
        Self(Box::new(inner), 0)
    }

    pub fn zero<T: UIWidget + 'static>(inner: T) -> Self {
        Self(Box::new(inner), 1)
    }

    pub fn one<T: UIWidget + 'static>(inner: T) -> Self {
        Self(Box::new(inner), 2)
    }

    pub fn two<T: UIWidget + 'static>(inner: T) -> Self {
        Self(Box::new(inner), 3)
    }

    pub fn three<T: UIWidget + 'static>(inner: T) -> Self {
        Self(Box::new(inner), 4)
    }

    pub fn four<T: UIWidget + 'static>(inner: T) -> Self {
        Self(Box::new(inner), 5)
    }

    pub fn five<T: UIWidget + 'static>(inner: T) -> Self {
        Self(Box::new(inner), 6)
    }
}

impl UIWidget for ZIndex {
    fn can_inherit(&self) -> bool {
        true
    }

    fn base_class(&self) -> Vec<String> {
        let class = match self.1 {
            0 => "z-auto",
            1 => "z-0",
            2 => "z-10",
            3 => "z-20",
            4 => "z-30",
            5 => "z-40",
            6 => "z-50",
            _ => "z-auto",
        };

        vec![class.to_string()]
    }

    fn extended_class(&self) -> Vec<String> {
        let mut c = self.base_class();
        c.extend_from_slice(&self.0.extended_class());
        c
    }

    fn render_with_class(&self, class: &str) -> Markup {
        if self.0.as_ref().can_inherit() {
            self.0
                .as_ref()
                .render_with_class(&format!("{} {class}", self.base_class().join(" ")))
        } else {
            html! {
                div class=(format!("{} {class}", self.base_class().join(" "))) {
                    (self.0.as_ref())
                }
            }
        }
    }
}
